@@ -1,13 +1,16 @@
 # Verifie les parametres d entrees
-CheckInputs <- function(x, g, initModel, vbleSelec, discrim, paramEstim, nbcores, nbSmall, iterSmall, nbKeep, iterKeep, tolKeep){
+CheckInputs <- function(x, g, initModel, vbleSelec, crit.varsel, discrim, paramEstim, nbcores, nbSmall, iterSmall, nbKeep, iterKeep, tolKeep){
   if ( (is.numeric(g)==FALSE) || (length(g)!=1))
     stop("The component number have to be an integer of length one!")
   
-  if ((is.data.frame(x)==FALSE) && (is.matrix(x)==FALSE))
-    stop("Data set must be a data frame or a matrix!")
+  if (is.data.frame(x)==FALSE)
+    stop("Data set must be a data frame!")
   
   if (is.logical(vbleSelec) == FALSE)
-    stop("Input vbleSelec must be logical")
+    stop("Input vbleSelec must be logicial")
+  
+  if ((crit.varsel %in% c("AIC", "BIC", "MICL"))==FALSE)
+    stop("Input vbleSelec must be equal to AIC, BIC or MICL")
   
   if ((length(discrim) != ncol(x)) || (all(discrim %in% c(0,1))==FALSE))
     stop("Input discrim must be logical of length number of variables")
